#!/bin/bash
SOURCE="$0"
source /usr/lib/elive-tools/functions
EL_REPORTS="1"
el_make_environment
# gettext functions
if [[ -x "/usr/bin/gettext.sh" ]] ; then
    . gettext.sh
else
    # make it compatible
    eval_gettext(){
        echo "$@"
    }
fi
TEXTDOMAIN="deliver"
export TEXTDOMAIN

# if we are in e16, our updated (reconfigured) language must be updated, that's not needed in e17 since the ENV var is set
if [[ -n "$E_ROOT" ]] ; then
    source /etc/default/locale 2>/dev/null || true
fi


main(){
    # pre {{{
    local message_wait
    message_wait="$( printf "$( eval_gettext "Please wait" )" "" )"

    # debug mode
    if grep -Fqs "debug" /proc/cmdline ; then
        export EL_DEBUG=3
        if grep -Fqs "completedebug" /proc/cmdline ; then
            set -x
        fi
    fi

    # show this in the already-opened terminal
    echo -e "\n$message_wait\n\n"

    # debug purposes only:
    #zenity --info --text="$(pwd) - $0 | $DISPLAY | $SHELL | $LANG | $message_wait" || true

    # }}}

    if grep -Fqs "no soundcards ---" /proc/asound/cards ; then
        ( $guitool --warning --text="$( eval_gettext "No audio cards detected in your system. We suggest trying a different kernel version. You can select the other kernel from the first menu that appears when booting Elive." )" ) &
    else

        el_debug "Configuring audio card..."

        if [[ ! -e "$HOME/.asoundrc" ]] || [[ ! -e "$HOME/.pulse" ]] || [[ ! -e "$HOME/.config/pulse" ]] ; then
            # we need to configure it as user too, because some things are triggered too, like vokoscreen configurations
            # note: configurator and volumes needs to be always set in order to configure correctly the relative apps too, this hook is only launched once per new desktop so should be not a problem at all
            audio-configurator --quiet --smart --auto --no-messages

            # now, and the only moment we set the volumes
            el_debug "Set default volumes..."
            setvolume defaults
            # decrease them because we are in Live
            setvolume 60%

            # now store the PCM (and volumes) channel so that we can have it among reloads
            el_debug "Store volume configurations..."
            sudo -H alsactl store

            # copy it to root too
            sudo -H cp -f "$HOME/.asoundrc" /root/.asoundrc
        fi


        # play elive startup sound (forced mode for live/persistence, after to have set the volumes)
        if [[ -x "/usr/bin/elive-startup-sound" ]] ; then
            ( elive-startup-sound -l & )
        fi

        # if we are in development we want to see debug lines in the opened terminal
        if el_check_version_development ; then
            echo -e "Done"
        fi
    fi

    # if we are debugging give it a little pause to see what is going on
    #if grep -Fqs "debug" /proc/cmdline ; then
        #echo -e "debug: sleep 2" 1>&2
        #sleep 2
    #fi
}

#
#  MAIN
#
main "$@"

# vim: set foldmethod=marker :

