#!/bin/bash
SOURCE="$0"
source /usr/lib/elive-tools/functions
EL_REPORTS="1"
#el_make_environment
. gettext.sh
TEXTDOMAIN="deliver"
export TEXTDOMAIN

# if we are in e16, our updated (reconfigured) language must be updated, that's not needed in e17 since the ENV var is set
if [[ -n "$E_ROOT" ]] ; then
    source /etc/default/locale 2>/dev/null || true
fi




main(){
    # pre {{{
    local memory cpubogomips

    # debug mode
    if grep -Fqs "debug" /proc/cmdline ; then
        export EL_DEBUG=3
        if grep -Fqs "completedebug" /proc/cmdline ; then
            set -x
        fi
    fi

    memory="$( el_ram_usage_show total )"
    cpubogomips="$( grep -F bogomips /proc/cpuinfo | sed -e 's|^.*: ||g' -e 's|\..*$||g' | tr ' ' '\n' | grep "[[:digit:]]" | head -1 )"

    # }}}

    # at least 1,2 gb of memory (useless to cache in less than that), and 4 GB cpu
    #if [[ "${memory}" -lt "650" ]] || [[ "$cpubogomips" -lt 2400 ]] ; then
    if [[ "${memory}" -lt "650" ]] ; then
        sudo -H service fail2ban stop 2>/dev/null
        sudo -H service samba stop 2>/dev/null
        sudo -H service cups stop 2>/dev/null
        sudo -H service rsyslog stop 2>/dev/null
        sudo -H service bluetooth stop 2>/dev/null
        sudo -H service dnsmasq stop 2>/dev/null
        sudo -H service gpm stop 2>/dev/null
        sudo -H service ssh stop 2>/dev/null
        # note: never disable udisks daemon, or we will have swapfiles broken (umounted partitions)
        sudo -H service mdadm stop 2>/dev/null

        sudo -H killall enlightenment-wallpapers-switcher 2>/dev/null


        # umount overlay {{{
        eliveuser="eliveuser"

        if grep -qsE "(^7\.|wheezy)" /etc/debian_version ; then
            LIVE_MOUNT_MAIN_d="/lib/live/mount"
        else
            LIVE_MOUNT_MAIN_d="/run/live"
        fi
        if ! [[ -d "$LIVE_MOUNT_MAIN_d" ]] ; then
            if grep -Fqs "live-media-path=" /proc/cmdline ; then
                LIVE_MOUNT_MAIN_d="$( cat /proc/cmdline | tr ' ' '\n' | grep "^live-media-path=" | sed -e 's|live-media-path=||g' | tail -1 )"
                if ! [[ -d "$LIVE_MOUNT_MAIN_d" ]] ; then
                    unset LIVE_MOUNT_MAIN_d
                fi
            fi
        fi
        if [[ -d "$LIVE_MOUNT_MAIN_d/overlay/rw" ]] ; then
            LIVE_MOUNT_OVERLAY_d="$LIVE_MOUNT_MAIN_d/overlay/rw"
        else
            LIVE_MOUNT_OVERLAY_d="$LIVE_MOUNT_MAIN_d/overlay"
        fi

        if ! el_check_dir_has_files "$LIVE_MOUNT_OVERLAY_d" 2>/dev/null ; then
            # this is a normal behaviour, we need to umount it one time to show the files (seems like its mounted twice)
            umount "${LIVE_MOUNT_OVERLAY_d}"  2>/dev/null || true
            if ! el_check_dir_has_files "$LIVE_MOUNT_OVERLAY_d" 2>/dev/null ; then
                umount "${LIVE_MOUNT_OVERLAY_d}"  2>/dev/null || true
                if ! el_check_dir_has_files "$LIVE_MOUNT_OVERLAY_d" 2>/dev/null ; then
                    umount "${LIVE_MOUNT_OVERLAY_d}"  2>/dev/null || true
                fi
            fi

            # if we are using persistence, the overlay could have been moved here
            if ! el_check_dir_has_files "$LIVE_MOUNT_OVERLAY_d" 2>/dev/null ; then
                if ls $LIVE_MOUNT_MAIN_d/persistence/*/persistence.conf  1>/dev/null 2>&1 ; then
                    LIVE_MOUNT_OVERLAY_d="$( find $LIVE_MOUNT_MAIN_d/persistence/*/persistence.conf | sed -e 's|/persistence.conf||g' | sort -u | head -1 )"
                    LIVE_MOUNT_OVERLAY_d="$( cat /proc/mounts | grep -F "${LIVE_MOUNT_OVERLAY_d}" | awk '{print $1}' | tail -1 )"
                fi
            fi
            if ! el_check_dir_has_files "$LIVE_MOUNT_OVERLAY_d" 2>/dev/null ; then
                if ls $LIVE_MOUNT_MAIN_d/persistence/*/etc/default/  1>/dev/null 2>&1 ; then
                    LIVE_MOUNT_OVERLAY_d="$( find $LIVE_MOUNT_MAIN_d/persistence/*/etc/default/ | sed -e 's|/etc/default.*$||g' | sort -u | head -1 )"
                    LIVE_MOUNT_OVERLAY_d="$( cat /proc/mounts | grep -F "${LIVE_MOUNT_OVERLAY_d}" | awk '{print $1}' | tail -1 )"
                fi
            fi
        fi

        # }}}

        # cleanup temporal unneeded files
        find "${LIVE_MOUNT_OVERLAY_d}/var/lib/apt/lists" -type f -delete
        find "${LIVE_MOUNT_OVERLAY_d}/var/lib/samba" -type f -delete
        find "${LIVE_MOUNT_OVERLAY_d}/var/cache/apt/" -type f -delete

        # browsers cleanup
        killall google-chrome  2>/dev/null
        killall chromium 2>/dev/null
        killall firefox-esr 2>/dev/null
        sync
        find "${LIVE_MOUNT_OVERLAY_d}/home/${eliveuser}/.cache/mozilla/" -type f -delete
        find "${LIVE_MOUNT_OVERLAY_d}/home/${eliveuser}/.cache/chromium/" -type f -delete
        find "${LIVE_MOUNT_OVERLAY_d}/home/${eliveuser}/.cache/google-chrome/" -type f -delete

        # trash
        find "${LIVE_MOUNT_OVERLAY_d}/home/${eliveuser}/.local/share/Trash/" -type f -delete
    fi

    # set terminology as default terminal (because is not by default)
    if [[ "${memory}" -gt "1200" ]] && [[ "$cpubogomips" -gt 1800 ]] && [[ -x "$(which terminology)" ]] ; then
        sudo -H update-alternatives --set x-terminal-emulator /usr/bin/terminology
    else
        if [[ "$( which urxvt )" ]] ; then
            sudo -H update-alternatives --set x-terminal-emulator /usr/bin/urxvt
        fi
    fi

    # warn about 32bit limitation ram
    if [[ "$( uname -m )" = "i"*"86" ]] ; then
        if [[ "${memory}" -gt "2100" ]] && [[ "${memory}" -lt "3200" ]] ; then
            zenity --warning --text="$( eval_gettext "Warning: This version of Elive is 32-bit, so it can only utilize up to 3.5GB of RAM. We recommend using the 64-bit version of Elive for better performance on your computer." )" &
        fi
    fi


    # clean logs, they are big
    # if sudo -H -n systemctl status systemd-journald.service 2>&1 | colors-remove | grep -qsi loaded ; then
    #     if [[ "${memory}" -lt "2300" ]] ; then
    #         sudo -H -n systemctl stop     systemd-journald.service systemd-journald.socket systemd-journald-audit.socket systemd-journald-dev-log.socket   1>/dev/null 2>&1 || true
    #         sudo rm -rf /var/log/journal/* 1>/dev/null 2>&1 || true
    #
    #     else
    #         # normal, just cleanup
    #         sudo -H -n systemctl stop     systemd-journald.service systemd-journald.socket systemd-journald-audit.socket systemd-journald-dev-log.socket   1>/dev/null 2>&1 || true
    #         sudo rm -rf /var/log/journal/* 1>/dev/null 2>&1 || true
    #         sudo -H -n systemctl start     systemd-journald.service systemd-journald.socket systemd-journald-audit.socket systemd-journald-dev-log.socket   1>/dev/null 2>&1 || true
    #     fi
    # else
    #     sudo rm -rf /var/log/journal/* 1>/dev/null 2>&1 || true
    # fi

    # enable services if they are worth
    if [[ "${memory}" -gt "1900" ]] ; then
        # enable samba
        sudo -H -n systemctl unmask    smbd.service  1>/dev/null 2>&1 || true
        sudo -H -n systemctl enable    smbd.service  1>/dev/null 2>&1 || true
        sudo -H -n systemctl start    smbd.service  1>/dev/null 2>&1 || true
    fi

    # reload confs
    sudo -H -n systemctl daemon-reload
}

#
#  MAIN
#
main "$@"

# vim: set foldmethod=marker :
