#!/bin/bash
SOURCE="$0"
source /usr/lib/elive-tools/functions
EL_REPORTS="1"
el_make_environment

TEXTDOMAIN="deliver"
export TEXTDOMAIN

# if we are in e16, our updated (reconfigured) language must be updated, that's not needed in e17 since the ENV var is set
if [[ -n "$E_ROOT" ]] ; then
    source /etc/default/locale 2>/dev/null || true
fi

#. gettext.sh
if [[ -x "/usr/bin/gettext.sh" ]] ; then
    . gettext.sh
else
    # make it compatible
    eval_gettext(){
        echo "$@"
    }
fi



main(){
    # pre {{{
    local var

    # debug mode
    if grep -Fqs "debug" /proc/cmdline ; then
        export EL_DEBUG=3
        if grep -Fqs "completedebug" /proc/cmdline ; then
            set -x
        fi
    fi

    # }}}
    elive_version="$( cat /etc/elive-version | grep "^elive-version:" | awk '{print $2}' )"
    case "$elive_version" in
        *.7.*)  is_alpha=1 ; ;;
        *.8.*)  is_beta=1 ; ;;
        *.9.*)  is_rc=1 ; ;;
        *.0.*)  is_stable=1 ; ;;
    esac

    # issues known report
    if ((is_alpha)) ; then
        if [[ -n "$E17_TAINTED" ]] ; then
            zenity --warning --text="$( eval_gettext "Issues known for these versions: some desktop modules doesn't work, like temperature or the battery. The desktop performance is very bad, feeling slow and lagged. The systray doesn't show the applications, making it difficult to configure your Wifi or using other applications. Special effects doesn't works. The compositor has some issues displaying the menus of applications or using a web browser. Right now, E16 is a more usable and suggested desktop.
            " )"
        fi

    fi

    if ! ((is_stable)) ; then
        if [[ -e "/tmp/.secureboot-enabled" ]] ; then
            #el_warning "SECUREBOOT ENABLED: \nvboxdrv: $(sudo modprobe vboxdrv 2>&1 )\nr8168: $(sudo modprobe r8168 2>&1 )"
            $guitool --warning --text="$( eval_gettext "Warning: Your computer won't work properly because of SecureBoot, we recommend turning it off in your BIOS settings, otherwise, some WiFi drivers, VirtualBox, and Nvidia graphics drivers might not function, you can find many guides online to help you disable SecureBoot." )"

        fi
    fi

    # update: annoying, and we will have already other similar messages, also the downloads page will require a donation for the 64bit versions maybe
    #if ((is_alpha)) ; then
        ## ask for funding support or promoting
        #if zenity --question --text="$( eval_gettext "These versions of Elive are not yet ready for the daily use and they has a lot of work remaining to be done for Elive. Donations are needed to continue the development. Do you want to participate?" )" ; then
            #web-launcher "https://www.patreon.com/elive"
        #else
            #zenity --info --text="$( eval_gettext "The development is right now paused due to low funding, but if you can help make Elive more known, this is even more important than donations." )"
        #fi
    #fi

    # always disable screen blanking by default
    xset dpms force on
    xset dpms 0 0 0


    # cleanups
    # free some memory by unneeded efreetd
    # update: probably not needed if pre-populated on build-time
    #if [[ -n "$EROOT" ]] ; then
        #if ! readlink -f /etc/alternatives/x-terminal-emulator | grep -Fqs "terminology" ; then
            #killall efreetd 1>/dev/null 2>&1 || true
            #killall -9 efreetd 1>/dev/null 2>&1 || true
            #rm -rf "$HOME/.cache/efreet"
        #fi
    #fi
}

#
#  MAIN
#
main "$@"

# vim: set foldmethod=marker :
