#!/bin/bash
SOURCE="$0"
source /usr/lib/elive-tools/functions
source /etc/elive/machine-profile 2>/dev/null || true
EL_REPORTS="1"

main(){
    #
    #
    # NOTE : disabled for now, we want to keep the system light in ram so don't run this one
    #
    #
    exit



    # pre {{{
    local file LIVE_MOUNT_MEDIA_DEV_v LIVE_MOUNT_MEDIA_PART_v LIVE_MOUNT_MAIN_d LIVE_MOUNT_ROOT_d LIVE_MOUNT_MEDIA_d is_live_from_usb resolution

    # debug mode
    if grep -Fqs "debug" /proc/cmdline ; then
        export EL_DEBUG=3
        if grep -Fqs "completedebug" /proc/cmdline ; then
            set -x
        fi
    fi

    # do not run in these cases
    if [[ "$MACHINE_VIRTUAL" = "yes" ]] || [[ -f "/tmp/.precaches-done" ]] ; then
        exit
    fi

    # determine the mount version
    if grep -qsE "(^7\.|wheezy)" /etc/debian_version ; then
        LIVE_MOUNT_MAIN_d="/lib/live/mount"
    else
        LIVE_MOUNT_MAIN_d="/run/live"
    fi
    #LIVE_MOUNT_ROOT_d="$LIVE_MOUNT_MAIN_d/rootfs/filesystem.squashfs"

    LIVE_MOUNT_MEDIA_d="$LIVE_MOUNT_MAIN_d/medium"
    LIVE_MOUNT_MEDIA_PART_v="$( LC_ALL=C mount | grep -F "$LIVE_MOUNT_MEDIA_d" | sed -e 's| on .*$||g' | head -1 )"
    LIVE_MOUNT_MEDIA_DEV_v="${LIVE_MOUNT_MEDIA_PART_v%%[0-9]*}"

    if LC_ALL=C udevadm info --query=all --name="$LIVE_MOUNT_MEDIA_DEV_v" 2>/dev/null | grep -Fqs "E: ID_USB_DRIVER=usb-storage" ; then
        is_live_from_usb=1
    fi
    # }}}
    # disable this feature in low memory computers
    local memory cpubogomips

    memory="$( el_ram_usage_show total )"
    cpubogomips="$( grep -F bogomips /proc/cpuinfo | sed -e 's|^.*: ||g' -e 's|\..*$||g' | tr ' ' '\n' | grep "[[:digit:]]" | head -1 )"

    # ignore in low-memory computers
    # at least 1,2 gb of memory (useless to cache in less than that), and 1.2 GHz cpu
    if [[ "${memory}" -lt "1400" ]] || [[ "$cpubogomips" -lt 2400 ]] ; then
        return 0
    fi


    # low disk priorization from here
    ionice -c3 -p$$


    source /etc/elive/machine-profile 2>/dev/null || true

    if ! ((is_live_from_usb)) || [[ "$MACHINE_VIRTUAL" = "yes" ]] ; then
        el_debug "ignoring precaches because we are not running from usb or we are in a virtual machine"
        exit
    fi


    # use precache command, it saves up to 3x times
    # use it in a one-shot combo for avoid duplicated libs (optimized)
    LC_ALL=C precache --nice /usr/bin/terminology /usr/bin/urxvt /usr/bin/Thunar /opt/google/chrome/chrome /usr/lib/chromium/chromium /usr/bin/vlc /usr/bin/nm-applet /usr/bin/audacious /usr/bin/gimp /usr/bin/inkscape /usr/bin/blender /usr/share/blender-last/blender "$( readlink -f "$( which iceweasel )" 2>/dev/null || true )" "$( readlink -f "$( which firefox )" 2>/dev/null || true )" "$( readlink -f "$( which google-chrome )" 2>/dev/null || true )" "$( readlink -f "$( which chromium )" 2>/dev/null || true )" 1>/dev/null


    if ! [[ -x /usr/bin/xvfb-run ]] ; then
        # libreoffice caching is not enough, let's use this way instead, so it will be closed after a while
        if [[ "${memory}" -gt "3400" ]] ; then
            timeout 40 libreoffice --invisible
        fi
    else
        # The previous method improved it 3 times:p
        # thunar    3,4 VS 2,7  seconds
        # chromium  9.3 VS 3,6  seconds
        # but we want better! so:

        resolution="$(el_resolution_get)"
        if ! echo "$resolution" | grep -qsE "[[:digit:]].*x[[:digit:]]" ; then
            resolution="1024x768"
        fi

        #LC_ALL=C precache --nice /usr/bin/nm-applet

        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 6 terminology 2>/dev/null  # disabled, because it generates some efreet stuff
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 6 nm-applet 2>/dev/null
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 20 thunar 2>/dev/null
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 32 /etc/alternatives/x-www-browser 2>/dev/null

        # chromium will generate .cache files that will use some of our RAM (2 MB in .cache and 1.4 MB in .config)
        # UPDATE: do not cache chrome! it hangs even with the timeout included hummm
        #if [[ -x /usr/bin/chromium ]] ; then
            #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 32 chromium --temp-profile 2>/dev/null
        #else
            #if [[ -x /usr/bin/google-chrome ]] ; then
                #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 34 google-chrome 2>/dev/null
            #else
                #if [[ -x /etc/alternatives/x-www-browser ]] ; then
                    #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 28 /etc/alternatives/x-www-browser 2>/dev/null
                #fi
            #fi
        #fi
        # note: audacious can't run twice (two apps at the same time, so if is running it may not run)
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 8 audacious 2>/dev/null
        #LC_ALL=C precache  /usr/bin/audacious
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 44 gimp -b "(gimp-quit 0)" 2>/dev/null
        xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 50 libreoffice 2>/dev/null
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 14 vlc 2>/dev/null
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 36 inkscape 2>/dev/null
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout 22 blender 2>/dev/null
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout
        #xvfb-run -a --server-args="-screen 0 ${resolution}x24"  timeout

        # mark pass
        touch /tmp/.precaches-done

    fi

    el_debug "finished caching apps"

}

#
#  MAIN
#
main "$@"

# vim: set foldmethod=marker :
