#!/bin/bash
SOURCE="$0"
source /usr/lib/elive-tools/functions
EL_REPORTS="1"
el_make_environment
. gettext.sh
TEXTDOMAIN="deliver"
export TEXTDOMAIN


main(){
    # pre {{{
    local file data line

    # debug mode
    if grep -Fqs "debug" /proc/cmdline ; then
        export EL_DEBUG=3
        if grep -Fqs "completedebug" /proc/cmdline ; then
            set -x
        fi
    fi

    mkdir -p "/tmp/preseeds.d"

    if [[ -e /tmp/.language-changed ]] ; then
        el_debug "already changed / configured language, exiting..."
        exit
    fi

    # do not ask again for langauges when persistence
    if [[ -r "/etc/default/persistence-elive.sh" ]] ; then
        source "/etc/default/persistence-elive.sh"
        if [[ -n "$counter_times_persistence_booted" ]] && [[ "${counter_times_persistence_booted}" -ge "1" ]] ; then
            el_debug "do not asking again for languages when we are running in persistence mode"
            exit
        fi
    fi

    if grep -Fqs "thanatests" /proc/cmdline ; then
        exit
    fi

    # }}}
    # get / ask language {{{
    # e16:
    if [[ -n "$EROOT" ]] ; then
        # ask for preselected language to use
        language="$LANG"
        language="${language%% *}"
        language="${language%%.*}"

        # wait for the first window to appear after to have reloaded the desktop, also the dpi is not yet set
        precache zenity
        sleep 2

        result="$( { echo "Other" ; echo "${language}" ; cat /etc/locale.gen | grep -vE "(^#|@)" | grep -F "UTF-8 UTF-8" | awk '{print $1}' | sed -E -e 's|\.UTF-8.*$||g' -e 's|^\s+||g' ; } | sort -u | psort -- -p "Other" -p "_${language##*_}$" -p "^${language%%_*}_" | zenity --width=180 --height=380 --list --text="Select your LANGUAGE.\nYour actual one is '${language}'" --column="language__COUNTRY" 2>/dev/null || echo cancel )"

        if [[ "$result" = "cancel" ]] ; then
            is_lang_canceled=1
        fi

        if [[ -n "$result" ]] && [[ "$result" != "cancel" ]] && [[ "$result" != "Other" ]] ; then
            if [[ "$result" != "${language}" ]] ; then
                #is_lang_changed=1
                touch /tmp/.language-changed
            fi
            result="${result}.UTF-8"
            export LANG="$result"
        fi
    fi

    # e17
    if [[ -n "$E_START" ]] ; then
        LANG="$( enlightenment_remote -lang-get )"
        # remove extra leading blank chars
        read -r LANG <<< "$LANG"
        LANG="${LANG/.utf8/.UTF-8}"
    fi

    # fallback / reconfigure
    if [[ "${result}" = "Other" ]] || [[ -z "$LANG" ]] ; then
        urxvt -e bash -c "sudo -H -n dpkg-reconfigure locales"
        source /etc/default/locale

        if [[ -n "$LANG" ]] && [[ "$LANG" != "${language}" ]] ; then
            #is_lang_changed=1
            touch /tmp/.language-changed
        fi
    fi

    # - ask for locale }}}
    # get / ask keyboard {{{
    if [[ -n "$EROOT" ]] ; then

        # if our lang is like es_ES, set the keyboard simply to it
        if [[ "$( echo "${LANG%.*}" | awk -v FS="_" '{print toupper($1)}' )" = "$( echo "${LANG%.*}" | awk -v FS="_" '{print toupper($2)}' )" ]] ; then
            kbmap="$( echo "${LANG%.*}" | awk -v FS="_" '{print $1}' )"
        else
            kbmap="$( setxkbmap -print -verbose 10 | grep -F "layout:" | awk '{print $2}' )"
        fi


    result="$( { sed '/^! layout$/,/^ *$/!d;//d' "/usr/share/X11/xkb/rules/base.lst" | sed -E -e 's|^\s+||g' | sort -u | awk '{code = $1 ; $1 = "" ; print code"\n"$0}' | sed -e 's|^\s*||g'   ; echo -e "Other\nConfigure" ; } | zenity --width=340 --height=380 --list --text="Select your KEYBOARD. Your actual one is '${kbmap}'" --column="Code" --column="Layout"  2>/dev/null || echo "cancel" )"
    # always set the keyboard to default when when we "validate" without selection:
    [[ -z "$result" ]] && [[ -n "$kbmap" ]] && result="$kbmap"

        if [[ "$result" = "cancel" ]] ; then
            result="$kbmap"
        else
            if [[ -n "$result" ]] && [[ "$result" != "cancel" ]] && [[ "$result" != "Other" ]] ; then
                result="$( echo "$result" | awk '{print $1}' )"

                # select a variant keyboard if needed
                XKBVARIANT="$( { echo -e "Normal\nNormal defaults for '$result'" ; sed '/^! variant$/,/^ *$/!d;//d' "/usr/share/X11/xkb/rules/base.lst" | awk -v lang="$result" '{if ($2 == lang":") print $0}' | sed -E -e 's|^\s+||g' | sort -u | awk '{$2 = "" ; code = $1 ; $1 = "" ; print code"\n"$0}' | sed -e 's|^  ||g' ; } | zenity --width=440 --height=380 --list --text="Select your variant for '${result}', or just Cancel" --column="Variant" --column="Description" 2>/dev/null )"
                XKBVARIANT="$( echo "$XKBVARIANT" | awk '{print $1}' )"

                if [[ -z "$XKBVARIANT" ]] || [[ "$XKBVARIANT" = "Normal"* ]] || ! echo "$XKBVARIANT" | grep -qs "[[:alpha:]]" ; then
                    unset XKBVARIANT
                fi
            fi
        fi

        # set US variant to altgr if empty
        if [[ -z "$XKBVARIANT" ]] && [[ "$result" = "us" ]] ; then
            XKBVARIANT="altgr-intl"
        fi

        # set the keyboard
        if [[ -n "$result" ]] && [[ "$result" != "cancel" ]] && [[ "$result" != "Other" ]] ; then
            XKBLAYOUT="$result"
            if ! setxkbmap "$XKBLAYOUT" -model "$XKBMODEL" -variant "$XKBVARIANT" -option "$XKBOPTIONS" ; then
                el_error "unable to set keyboard: setxkbmap '$XKBLAYOUT' -model '$XKBMODEL' -variant '$XKBVARIANT' -option '$XKBOPTIONS' : $( setxkbmap "$XKBLAYOUT" -model "$XKBMODEL" -variant "$XKBVARIANT" -option "$XKBOPTIONS" 2>&1 )"
            fi
        fi

        # fallback / reconfigure
        if [[ "${result}" = "Other" ]] ; then
            urxvt -e bash -c "sudo -H -n dpkg-reconfigure keyboard-configuration"
            source /etc/default/keyboard
            if ! setxkbmap "$XKBLAYOUT" -model "$XKBMODEL" -variant "$XKBVARIANT" -option "$XKBOPTIONS" ; then
                el_error "unable to set keyboard: setxkbmap '$XKBLAYOUT' -model '$XKBMODEL' -variant '$XKBVARIANT' -option '$XKBOPTIONS' : $( setxkbmap "$XKBLAYOUT" -model "$XKBMODEL" -variant "$XKBVARIANT" -option "$XKBOPTIONS" 2>&1 )"
            fi
        fi
    fi

    # Note: e17 is already configured by default

    # }}}

    # Get xkb values set {{{
    while read -ru 3 line
    do
        # layout
        if [[ "$line" = layout:* ]] ; then
            k_layout="${line#*:}"
            # do we have something useful?
            if echo "$k_layout" | sed -e 's|,||g' | grep -qs "[[:alpha:]]" ; then
                # remove last comma
                k_layout="${k_layout%,}"
                # remove extra leading blank chars
                read -r k_layout <<< "$k_layout"
            else
                unset k_layout
            fi
        fi

        # variant
        if [[ "$line" = variant:* ]] ; then
            k_variant="${line#*:}"
            # do we have something useful?
            if echo "$k_variant" | sed -e 's|,||g' | grep -qs "[[:alpha:]]" ; then
                # remove last comma
                k_variant="${k_variant%,}"
                # remove extra leading blank chars
                read -r k_variant <<< "$k_variant"
            else
                unset k_variant
            fi
        fi

        # options
        if [[ "$line" = options:* ]] ; then
            k_options="${line#*:}"
            # do we have something useful?
            if echo "$k_options" | sed -e 's|,||g' | grep -qs "[[:alpha:]]" ; then
                # remove extra leading blank chars
                read -r k_options <<< "$k_options"
            else
                unset k_options
            fi
        fi

    done 3<<< "$( setxkbmap -print -verbose 10 )"

    # - Get xkb values set }}}

    # always enable ctrl-alt-backspace to break X:
    # if [[ -n "$k_options" ]] ; then
    #     k_options="${k_options%,},terminate:ctrl_alt_bksp"
    # else
    #     k_options="terminate:ctrl_alt_bksp"
    # fi

    # configure language
    if [[ "$LANG" = *UTF* ]] || grep -qs "^${LANG} UTF-8" /etc/locale.gen ; then

        if ((is_lang_canceled)) ; then
            el_debug "ignoring reconfigure of languages, user canceled"
            is_user_ignored=1
        else
            el_debug "Configuring system language to $LANG..."
            sudo -H -s -- <<EOF
echo "LANG=$LANG" > /etc/default/locale
if ! grep -qs "^${LANG} ${LANG#*.}$" /etc/locale.gen ; then
    sed -i -e "s|# ${LANG} ${LANG#*.}|${LANG} ${LANG#*.}|" /etc/locale.gen
    locale-gen --keep-existing > /dev/null 2>&1
    update-locale
fi
echo -e "locales\tlocales/default_environment_locale\tselect\t${LANG}" > /tmp/preseeds.d/locales_default-environment-noreconfigure
echo -e "locales\tlocales/default_environment_locale\tselect\t${LANG}" >> /tmp/preseeds.cfg

# reconfigure console locale
dpkg-reconfigure -f noninteractive -p critical console-setup

# force reload configurations on console
setupcon --force

EOF

        fi
    else
        el_warning "Is your language not in UTF-8? ignoring language configuration: $LANG"
    fi

    # free up some space
    if ! ((is_user_ignored)) ; then
      el_debug "Free up some locales file space..."
      sudo -H -s -- <<EOF
if ! [[ -s /etc/locale.gen.orig ]] ; then
    mv -f /etc/locale.gen /etc/locale.gen.orig
fi
cat /etc/locale.gen.orig | grep -E "^(${LANG}|en_US)" > /etc/locale.gen
mv -f /usr/lib/locale/locale-archive /usr/lib/locale/locale-archive.tmpl
locale-gen --keep-existing > /dev/null 2>&1
rm -f /usr/lib/locale/locale-archive.tmpl
mv -f /etc/locale.gen.orig /etc/locale.gen
EOF
    fi

    source /etc/default/locale

    # configure xserver-xorg values, preseeds.d is run from the install with a dpkg-reconfigure for filename
    echo -e "xserver-xorg\txserver-xorg/config/inputdevice/keyboard/layout\tselect\t${k_layout%%,*}" > "/tmp/preseeds.d/xserver-xorg_kbd-layout"

    # configure keyboard
    el_debug "Configuring keyboard, layout  '${k_layout%%,*}' "
    el_debug "Configuring keyboard, variant '${k_variant%%,*}' "
    el_debug "Configuring keyboard, options '${k_options%%,*}' "

    sudo -H -s -- <<EOF
sed -i "s|^XKBLAYOUT.*$|XKBLAYOUT=\"${k_layout%%,*}\"|g" /etc/default/keyboard
sed -i "s|^XKBVARIANT.*$|XKBVARIANT=\"${k_variant%%,*}\"|g" /etc/default/keyboard
sed -i "s|^XKBOPTIONS.*$|XKBOPTIONS=\"${k_options}\"|g" /etc/default/keyboard
EOF

    # mark it to reconfigure for install
    echo "keyboard-configuration" >> /tmp/.packages-to-reconfigure
    echo "xserver-xorg" >> /tmp/.packages-to-reconfigure
    echo "console-setup" >> /tmp/.packages-to-reconfigure

    # if we are debugging give it a little pause to see what is going on
    #if grep -Fqs "debug" /proc/cmdline ; then
        #echo -e "debug: sleep 2" 1>&2
        #sleep 2
    #fi
    if [[ -e /tmp/.language-changed ]] ; then
        source /etc/default/locale

        if [[ -n "$EROOT" ]] ; then
            sync

            # restart desktop entirely
            zenity --info --text="$( eval_gettext "You changed the language, your desktop needs to be restarted." )"

            # clean user dirs confs
            rm -rf "$HOME/.config/user-dirs"*  "$HOME/.config/elive/migrator/xdg-default-dirs-language-upgraded.state"  1>/dev/null 2>&1 || true

            #eesh exit logout
            e17-restart-and-remove-conf-file-WARNING-dont-complain

            sync
            sleep 10
            exit
        fi
    fi
}

#
#  MAIN
#
main "$@"

# vim: set foldmethod=marker :
