#!/bin/bash
SOURCE="$0"
source /usr/lib/elive-tools/functions
EL_REPORTS="1"
#el_make_environment
# gettext functions
if [[ -x "/usr/bin/gettext.sh" ]] ; then
    . gettext.sh
else
    # make it compatible
    eval_gettext(){
        echo "$@"
    }
fi

TEXTDOMAIN="deliver"
export TEXTDOMAIN

# if we are in e16, our updated (reconfigured) language must be updated, that's not needed in e17 since the ENV var is set
if [[ -n "$E_ROOT" ]] ; then
    source /etc/default/locale 2>/dev/null || true
fi


main(){
    # pre {{{
    local var

    # debug mode
    if grep -Fqs "debug" /proc/cmdline ; then
        export EL_DEBUG=3
        if grep -Fqs "completedebug" /proc/cmdline ; then
            set -x
        fi
    fi

    # determine the mount version
    case "$( cat /etc/debian_version )" in
        12.*|"bookworm"*)
            is_bookworm=1
            ;;
        11.*|"bullseye"*)
            is_bullseye=1
            ;;
        10.*|"buster"*)
            is_buster=1
            ;;
        7.*|"wheezy"*)
            is_wheezy=1
            LIVE_MOUNT_MAIN_d="/lib/live/mount"
            ;;
    esac

    if grep -Fqs "thanatests" /proc/cmdline ; then
        is_thanatests=1
    fi


    # }}}
    source /etc/elive/machine-profile 2>/dev/null || true

    if [[ "$MACHINE_VIRTUAL_TYPE" = "virtualbox" ]] && ! ((is_thanatests)) ; then
        if glxinfo 2>&1 | grep -qsiE "(vendor string: Chromium|Vendor: VMware, Inc|Device: SVGA3D)" ; then
            is_3d_enabled=1
        fi

        # only E17 ?
        if [[ "$E_HOME_DIR" = "$HOME/.e/e17" ]] ; then
            if ! el_flag check "vbox-3d-warn" ; then

                if ((is_3d_enabled)) ; then
                    # chromium is the default 3d enabled mode (if not it says SGI, Mesa, and VMware ones)
                    timeout 90 zenity --warning --text="$( eval_gettext "Disable the 3D option in your Virtual Machine's Display Settings. Shutdown it and disable this option to prevent issues like a black screen or desktop artifacts." )" &
                    el_flag add "vbox-3d-warn"
                fi
            fi
        fi

        # e16 + e26
        if [[ -n "$EROOT" ]] || [[ "$E_HOME_DIR" = "$HOME/.e/e" ]] ; then
            if ((is_bullseye)) || ((is_bookworm)) ; then
                if ! ((is_3d_enabled)) ; then
                    timeout 90 zenity --warning --text="$( eval_gettext "Enable the 3D option in your Virtual Machine's Display Settings to avoid desktop restarts when running applications like the Thunar or VokoScreen." )" &

                    # temporal fix for thunar: XXX FIXME DELETEME
                    sudo rm -f /usr/lib/*/tumbler*/plugins/tumbler-gst-thumbnailer.so
                fi
            else
                el_warning "TODO: does the desktop still crashing when running vokoscreen on vbox without the 3d option enabled?"
            fi
        fi

    fi

}

#
#  MAIN
#
main "$@"

# vim: set foldmethod=marker :
