#!/bin/bash
SOURCE="$0"
source /usr/lib/elive-tools/functions
EL_REPORTS="1"
#el_make_environment
unset E_START
# gettext functions
if [[ -x "/usr/bin/gettext.sh" ]] ; then
    . gettext.sh
else
    # make it compatible
    eval_gettext(){
        echo "$@"
    }
fi
TEXTDOMAIN="deliver"
export TEXTDOMAIN


source /etc/elive/machine-profile 2>/dev/null || true
source /etc/default/locale 2>/dev/null || true
# export variables sourced here and don't fail if the file is empty
set -a
for file in /etc/environment /etc/environment.d/*.conf
do
    source "$file" 2>/dev/null || true
done
set +a

# wrapper for fix strange languages
dialog(){
    if [[ "$LANG" = bg* ]] || [[ "$LANG" = bn* ]] || [[ "$LANG" = el* ]] || [[ "$LANG" = hi* ]] || [[ "$LANG" = ja* ]] || [[ "$LANG" = ko* ]] || [[ "$LANG" = ru* ]] || [[ "$LANG" = sr* ]] || [[ "$LANG" = te* ]] || [[ "$LANG" = uk* ]] || [[ "$LANG" = vi* ]] || [[ "$LANG" = zh* ]] ; then
        LANG=en_US.UTF-8  command dialog "$@"
    else
        command dialog "$@"
    fi
}

ask_for_console(){
    el_info "Select an option:\n"
    echo -e "1) Start the graphical system again"
    echo -e "2) Start a SHELL"
    echo -e "3) Exit this tool and access to the console"
    read -N 1 option

    case "$option" in
        1)  true ; ;;
        2)
            el_explain 0 "$message_report"
            el_explain 0 "$message_sudo"
            $SHELL
            ;;
        3)  exit ; ;;
    esac
}

# NOTE !!!  We are not root here, we are USER, make sure to run sudo to what is needed

# wait_for_remaining_processes(){
# # UPDATE: not needed anymore, systemd launcher has added a dependency to finish drivers-install before to run it
#     ## pre {{{
#     local count daemons i sleep_time
#
#     ## }}}
#
#     ## add all the init daemons of deliver installed
#     ## note: components of live-config seems to be only dumped to the /tmp/.live-processes-finished when we use systemd, not sysvinit
#     for i in $( cat /var/lib/dpkg/info/deliver.list | grep "^/etc/init.d/" | sed -e 's|^/etc/init.d/||g' | sort -u )
#     do
#         [[ ! -n "$i" ]] && continue
#         el_array_member_add "$i" "${daemons[@]}" ; daemons=("${_out[@]}")
#     done
#
#     if [[ -n "${daemons[@]}" ]] ; then
#         true
#     fi
#
#     count=0
#     #sleep_time=0.4
#     sleep_time=0.8
#     # timeout must be relative to the sleep time (0.5 = timeout x 2)
#     # 3 minutes
#     timeout="$(( 180 / $sleep_time ))"
#
#     echo
#
#     tput sc
#     #while fuser /var/lib/dpkg/lock >/dev/null 2>&1 ; do
#     while true ; do
#         # remove entries
#         for i in $( cat /tmp/.live-processes-finished 2>/dev/null | sed -e 's|^/etc/init.d/||g' -e 's|/lib/live/config/||g' -e 's|/lib/live/boot/||g' )
#         do
#             [[ ! -n "$i" ]] && continue
#             el_array_member_unset "$i" "${daemons[@]}" ; daemons=("${_out[@]}")
#         done
#         #
#         # no more processes remaining? we are done
#         if [[ -z "${daemons[@]}" ]] ; then
#             break
#         fi
#         if [[ "$count" -ge "$timeout" ]] ; then
#             el_warning "timeout ${0}:${FUNCNAME}:${LINENO} (timeout ${timeout}, sleep_time ${sleep_time})"
#             break
#         fi
#         #
#         case $(($count % 4)) in
#             0 ) j="-" ;;
#             1 ) j="\\" ;;
#             2 ) j="|" ;;
#             3 ) j="/" ;;
#         esac
#         tput rc
#         #
#         echo -en "\033[2K\r[$j] ${el_c_c}Almost ready:  "$( echo "${daemons[@]}" | tr '\n' ' ' )"${el_c_n}"
#         #echo -en "\033[2K\r[$j] ($count) Waiting processes to finish:  "$( echo "${daemons[@]}" | tr '\n' ' ' )""
#         #
#         # note: timeout must be double if 0.5
#         LC_ALL=C sleep $sleep_time
#         #LC_ALL=C sleep 1.0
#         #
#         ((count=count+1))
#     done
#
#     # just print a new line
#     echo
# }

state_valid_reached(){
    (
        # update: never modify /etc/elive-version, we need it untouched for persistence verifications
        # if ! grep -qs "^machine-id:" /etc/elive-version 2>/dev/null ; then
        #     sudo -H bash -c 'source /usr/lib/elive-tools/functions ; id="$( el_get_machine_id )" ; sed -i "/^machine-id:.*$/d" /etc/elive-version ; echo "machine-id: $id" >> /etc/elive-version'
        # fi
        # machine_id="$( grep "^machine-id: " /etc/elive-version 2>/dev/null | sed -e 's|^machine-id: ||g' | tail -1 )"
        machine_id="$( sudo -H bash -c 'source /usr/lib/elive-tools/functions ; id="$( el_get_machine_id )" ; echo "$id"' )"
        if [[ -z "$machine_id" ]] ; then
            machine_id="$( cat /etc/machine-id 2>/dev/null | tail -1 )"
        fi

        architecture="$( source /usr/lib/elive-tools/functions ; el_architecture "host" )"
        return_status=0
        eliveversion="$( awk '$1 ~ /elive-version/ {($1="");print $0}' /etc/elive-version | sed 's/^\ //g' )"

        source /etc/elive/machine-profile 2>/dev/null || true
        source /etc/elive/settings 2>/dev/null || true

        if [[ -n "$MACHINE_VIRTUAL" ]] ; then
            custom="machine_virtual"
        else
            custom="machine_real"
        fi

        # wait for a valid connection
        count=0
        while true ; do
            # max reached
            if [[ "$count" -gt 30 ]] ; then
                break
            fi
            # verify
            if el_verify_internet  fast  2>/dev/null 1>&2 ; then
                break
            else
                count="$(( $count + 1 ))"
                sleep 10
            fi
        done

        if el_verify_internet ; then
            sleep 1
            echo -e "ELIVE_DATA_TYPE: 1.0::${machine_id}::transfer::text::liveboot::0::$(whoami)::${conf_send_debug_reports_email}::${return_status}::${custom}::${architecture}::${eliveversion}::${SOURCE}::" | timeout 8 nc www.elivecd.org 60008 -w 6 1>/dev/null 2>&1
        fi
    )
}

info_release(){
    local stability
    local message_stability

    stability="$( grep -F "elive-version:" /etc/elive-version | awk '{print $2}' | tr '.' ' ' | awk '{print $2}' )"

    case "$stability" in
        7) stability="alpha"
            message_stability="$( printf "$( eval_gettext "This version is in ALPHA, indicating early development. Not everything may work. Use the stable release for a polished system, though it has older software. Stay updated via our forums!" )" "" )"
        ;;
        8) stability="beta"
            #message_stability="$( printf "$( eval_gettext "The stability status of this version is BETA, you should be able to use it as a stable system for your daily use, but some things may need to be polished. Be updated from our forums!" )" "" )"
            true
        ;;
        7) stability="rc"
            #message_stability="$( printf "$( eval_gettext "The stability status of this version is a Release-Candidate. This means that everything should be correctly working and also having a complete and intuitive usage. But if you see any issue is now the crucial momento to report it so we can resolve them for the final version and have another, excelent Elive historical release. If you want a final polished system use the stable release instead." )" "" )"
            true
        ;;
        #0) stability="stable"
            #message_stability="$( printf "$( eval_gettext "" )" "" )"
        #;;
        #*) stability="unknown"
            #message_stability="$( printf "$( eval_gettext "" )" "" )"
        #;;
    esac

    if [[ -n "$message_stability" ]] ; then
        dialog --timeout 30 --clear --colors --backtitle "Elive Systems: Use Ctrl-L to redraw, Enter to confirm" \
            --title "Elive Release Information" \
            --msgbox "$message_stability" \
            0 0 || true
    fi
}
select_windowmanager(){
    local packages
    source /etc/default/locale || true

    if [[ -x "$(which enlightenment)" ]] ; then
        local message_select_driver count
        message_select_driver="$( printf "$( eval_gettext "Choose your desktop. Each option offers different experiences and functionalities." )" "" )"

        # set the new E version
        e_new_version="E$( LC_ALL=C apt-cache show enlightenment 2>/dev/null | grep "^Version: " | awk '{print $2}' | sort -V | tail -1 | sed -e 's|-.*$||g' -e 's|^0\.||g' -e 's|\..*$||g')"

        (
            # E16
            _id="E16"
            _desc="$( printf "$( eval_gettext "Stable: Well integrated, retro style, fast, rock solid. Recommended option" )" )"
            echo -e "'$_id' '$_desc'"

            # E26+
            _id="$( LC_ALL=C apt-cache show enlightenment 2>/dev/null | grep "^Version: " | awk '{print $2}' | sort -V | tail -1 | sed -e 's|-.*$||g' -e 's|^0\.||g' -e 's|\..*$||g')"
            read -r _id <<< "$_id"
            if [[ -n "$_id" ]] ; then
                _id="E${_id}"
                _desc="$( printf "$( eval_gettext "Development: Newer, nicer, featured. But cutting edge and can have bugs." )" )"

                echo -e "'$_id' '$_desc'"
            fi

            #_id="E17"
            ##_desc="$( printf "$( eval_gettext "Not working good now. Tons of customizations and features for Elive, beautiful." )" )"
            #_desc="$( printf "$( eval_gettext "NOT WORKING GOOD. Beautiful. Intuitive. Was very good in the stable release." )" )"
            #echo -e "'$_id' '$_desc'"

        ) | xargs dialog --timeout 440 --clear --colors --backtitle "Enlightenment Desktops" --menu "$message_select_driver" 0 0 0  2>"/tmp/.$(basename $0)-ret"

        # user canceled? uninstall and exit
        if ! echo "$?" | grep -qsE "^(1|123|255|125)$" ; then
            # get result
            ret="$( cat "/tmp/.$(basename $0)-ret" | head -1 )"
            rm -f "/tmp/.$(basename $0)-ret"
            # remove extra leading blank chars
            read -r ret <<< "$ret"
        fi
    fi

    # if users's cancel, we have no results at all
    if [[ -z "$ret" ]] ; then
        if [[ -n "$e_new_version" ]] ; then
            ret="$e_new_version"
        fi

        if [[ -x "$(which e16)" ]] ; then
            ret="E16"
        fi
    fi

    case "$ret" in
        E17)
            if ! [[ -x "$( which e_wrapper )" ]] ; then
                packages="e17 e17-theme-elive-light elive-desktop-wrapper e17-conf e17-menus gnome-icon-theme"
                if ! sudo -H -n apt-get install -y $packages; then
                    sudo -H -n apt-get update
                    sudo -H -n apt-get install -y $packages || el_error "unable to install e17 (elive): $( timeout 80 sudo -H -n apt-get install -y $packages )"
                fi
                # save for the installed system
                for i in $packages
                do
                    echo "$i" >> /tmp/.packages-to-install
                done
            fi

            # precache
            # in bg because is bigger
            ( precache --nice  /usr/share/enlightenment/data/backgrounds/* /usr/bin/elementary_config /usr/share/elementary/themes/*edj /usr/share/applications/* /usr/share/xdgeldsk/applications/*  1>/dev/null  & )
            # not in bg
            precache  "$(which xinit 2>/dev/null)" "/usr/lib/xorg/Xorg" "$(which enlightenment_start 2>/dev/null)" "$(which enlightenment 2>/dev/null)" "$(which terminology 2>/dev/null)"  "$(which pulseaudio 2>/dev/null)" "$( which zenity 2>/dev/null )"  1>/dev/null

            # info
            if ! ((is_persistence)) && ! ((is_thanatests)) ; then
                local message_wm
                message_wm="$( printf "$( eval_gettext "WARNING: E17 integration has some issues, and certain features may not work correctly. E17 has been the default, highly customized desktop for Elive, featuring many capabilities. You should use E17 in the Elive 3.0 release, not this version." )" "" )"
                dialog --timeout 80 --clear --colors --backtitle "Elive Systems: Use Ctrl-L to redraw, Enter to confirm" \
                    --title "Desktop selected information" \
                    --msgbox "$message_wm" \
                    0 0 || true
            fi
            # select wm
            windowmanager="$( which e_wrapper )"
            ;;

        "$e_new_version")
            if ! ls /var/lib/dpkg/info/enlightenment-data*list 1>/dev/null 2>&1 ; then
                if ! sudo -H -n apt-get install -y enlightenment ; then
                    sudo -H -n apt-get update
                    sudo -H -n apt-get install -y enlightenment || el_error "unable to install enlightenment: $( timeout 80 sudo -H -n apt-get install -y enlightenment )"
                    is_e_updated=1
                fi
                # save for installed system
                echo "enlightenment" >> /tmp/.packages-to-install
            fi

            if ! ((is_e_updated)) && ! ((is_thanatests)) && el_verify_internet fast ; then
                local message_1
                message_1="$( printf "$( eval_gettext "Update the Desktop to the latest version made by Elive, it will use some RAM but will give you the newest features, updates, and fixes from Elive." )" "" )"

                dialog --timeout 90 --clear --colors --backtitle "Elive Systems: Use Ctrl-L to redraw, TAB to select, Enter to confirm" \
                    --title "$( eval_gettext "Update Enlightenment Desktop" )" \
                    --yesno "$message_1" \
                    0 0
                ret="$?"


                case "$ret" in
                    0) # yes

                        # ask if we want to restart a new desktop configuration
                        if ((is_persistence)) && [[ -e "$HOME/.e" ]] ; then
                            local message_1
                            message_1="$( printf "$( eval_gettext "Do you want to reset your configuration into a new, fresh one with the latest improvements?" )" "" )"

                            dialog --timeout 90 --clear --colors --backtitle "Elive Systems: Use Ctrl-L to redraw, TAB to select, Enter to confirm" \
                                --title "$( eval_gettext "Update Enlightenment Desktop" )" \
                                --yesno "$message_1" \
                                0 0
                            ret="$?"
                            if [[ "$ret" = "0" ]] ; then
                                rm -rf "$HOME/.e"
                            fi
                        fi

                        unset packages
                        packages="enlightenment enlightenment-data enlightenment-conf elive-tools elive-daemons elementary-theme-elive-light"
                        if dpkg -l | LC_ALL=C grep -qsE "^(ii|hi).*enlightenment-dev" ; then
                            packages="$packages enlightenment-dev"
                        fi
                        if dpkg -l | LC_ALL=C grep -qsE "^(ii|hi).*enlightenment-dbgsym" ; then
                            packages="$packages enlightenment-dbgsym"
                        fi
                        if el_package_upgrade_fast 1200 normal \
                            $packages \
                            ; then

                            is_e_updated=1
                        fi

                        ;;
                    1|255|123) # no, timeout, cancel
                        true

                        ;;
                esac
            fi

            # low stability requires updates:
            stability="$( grep -F "elive-version:" /etc/elive-version | awk '{print $2}' | tr '.' ' ' | awk '{print $2}' )"
            if [[ "$stability" != "stable" ]] ; then
                touch /tmp/.installation-full-upgrade-needed
            fi

            # select wm
            windowmanager="$( which enlightenment_start )"

            # precache
            # in bg because is bigger
            # ( precache --nice  /usr/share/enlightenment/data/backgrounds/* /usr/bin/elementary_config /usr/share/elementary/themes/*edj /usr/share/applications/* /usr/share/xdgeldsk/applications/*  1>/dev/null  & )
            # not in bg
            # precache  "$(which xinit 2>/dev/null)" "/usr/lib/xorg/Xorg" "$(which enlightenment_start 2>/dev/null)" "$(which enlightenment 2>/dev/null)" "$(which terminology 2>/dev/null)"  "$(which pulseaudio 2>/dev/null)" "$( which zenity 2>/dev/null )"  1>/dev/null
            ( precache --nice  /usr/share/enlightenment/data/backgrounds/* /usr/bin/elementary_config /usr/share/elementary/themes/*edj /usr/share/applications/* /usr/share/xdgeldsk/applications/*  1>/dev/null "$(which xinit 2>/dev/null)" "/usr/lib/xorg/Xorg" "$(which enlightenment_start 2>/dev/null)" "$(which enlightenment 2>/dev/null)" "$(which terminology 2>/dev/null)"  "$(which pulseaudio 2>/dev/null)" "$( which zenity 2>/dev/null )"  & )

            # prepare E for first run {{{

            # add some debug
            # export EINA_LOG_LEVEL=3

            # optimize useless search of eina_prefix:
            if [[ -d /usr/share/ecore ]] ; then
                export ECORE_PREFIX=/usr
            fi

            # set a configured dbus value to use
            if test -S "/run/user/$UID/bus" ; then
                export DBUS_SESSION_BUS_ADDRESS="unix:path=/run/user/$UID/bus"
            fi

            # set variables like XDG_DATA_DIRS
            if [ -d /etc/environment.d ] ; then
                . /etc/environment.d/*.conf 2>/dev/null || true
            fi
            . /etc/environment          2>/dev/null || true

            # tell cairo-dock to use our E menus
            export XDG_MENU_PREFIX="e-"

            # NOTE: do not enable/run efreetd, so it may conflict with the running E at start
            # ( killall -9 efreetd 1>/dev/null 2>&1 || true ; EINA_LOG_LEVEL=0 efreetd & )

            # }}}

            # info
            if ! ((is_persistence)) && ! ((is_thanatests)) ; then
                local message_wm
                #message_wm="$( printf "$( eval_gettext "WARNING: This newer version of Enlightenment is more beautiful but not yet fully integrated to use in Elive. If you wish to use it, you must customize many things of the desktop by yourself and manage any potential bugs. MANY ELIVE FEATURES WILL NOT FUNCTION. You are more than welcome to try it and to give us your feedback in our forums. If you want a stable environment for daily work you must use the E16 option instead which is fully integrated for Elive." )" "" )"
                message_wm="$( printf "$( eval_gettext "WARNING: This new version of the desktop is still in development and can have errors. It has more features but isn't fully compatible or integrated with Elive yet. If you want to use it, you'll need to customize a lot yourself and let us know about any bugs so we can fix them. Try it and share your feedback in our chat and forum. However, if you need a stable setup for daily work, it's best to stick with the E16 option, which is fully integrated with Elive." )" "" )"
                dialog --timeout 80 --clear --colors --backtitle "Elive Systems: Use Ctrl-L to redraw, Enter to confirm" \
                    --title "Desktop selected information" \
                    --msgbox "$message_wm" \
                    0 0 || true

                # crowdfunding
                message_wm="$( printf "$( eval_gettext "Elive is working hard integrating the new Enlightenment desktop! Join in the crowdfunding campaign to make it a reality sooner!" )" "" )"
                dialog --timeout 40 --clear --colors --backtitle "Elive Systems: Use Ctrl-L to redraw, Enter to confirm" \
                    --title "New Enlightenment Support" \
                    --msgbox "$message_wm" \
                    0 0 || true
            fi

            ;;

        E16|starte16)
            # fallback too
            if ! [[ -x "$(which e16)" ]] ; then
                packages="e16 e16-docs e16-epplets e16-theme-elivedark brightnessctl ulauncher pulseaudio conky lxappearance"
                if ! sudo -H -n apt-get install -y $packages ; then
                    sudo -H -n apt-get update
                    sudo -H -n apt-get install -y $packages || el_error "unable to install e16: $( timeout 80 sudo -H -n apt-get install -y $packages )"
                fi
                # save for installed system
                for i in $packages
                do
                    echo "$i" >> /tmp/.packages-to-install
                done
            fi

            # precache it for a fast startup
            # in bg because is bigger
            ( precache --nice  /usr/share/e16/themes/*/sound/samples/* /usr/share/applications/* /usr/share/xdgeldsk/applications/* "$(which ulauncher 2>/dev/null)"  1>/dev/null  & )
            # not in BG
            precache  "$(which xinit 2>/dev/null)" "/usr/lib/xorg/Xorg" "$(which e16 2>/dev/null)" "$(which pulseaudio 2>/dev/null)" "$( which zenity 2>/dev/null )"  1>/dev/null

            # info
            if ! ((is_persistence)) && ! ((is_thanatests)) ; then
                local message_wm
                message_wm="$( printf "$( eval_gettext "E16 offers an extremely stable desktop and is perfect for daily work. Elive managed to create a pretty full-featured desktop with E16. The look can feel retro but still be extremely useful, fast, and light. We hope you will enjoy this unique experience and don't forget to try our coolest selection of themes included. Allowing you to customize your desktop to your best own taste!" )" "" )"
                dialog --timeout 60 --clear --colors --backtitle "Elive Systems: Use Ctrl-L to redraw, Enter to confirm" \
                    --title "Desktop selected information" \
                    --msgbox "$message_wm" \
                    0 0 || true
            fi
            # remove edox from live since is more annoying than other thing
            sudo rm -f /usr/bin/edox
            # select wm
            windowmanager="$(which starte16)"
            ;;

        # by default, we want to use E16 (previously set fallback)
        *)
            # fallback
            if [[ -e "/etc/alternatives/x-window-manager" ]] ; then
                windowmanager="/etc/alternatives/x-window-manager"
            else
                if [[ -e "$( which starte16 )" ]] ; then
                    windowmanager="$(which starte16)"
                else
                    windowmanager="$(which enlightenment_start)"
                fi
            fi
            echo -e "\n\n\n"
            el_warning "Unknown window manager selected: $ret"
            sleep 1
            ;;
    esac

    # set the wanted WM to be default on the OS:
    sudo -H update-alternatives  --remove-all x-window-manager 1>/dev/null 2>&1 || true
    sudo -H update-alternatives  --install /usr/bin/x-window-manager x-window-manager "$windowmanager" 500 1>/dev/null 2>&1
}

main(){
    # pre {{{
    local _CMDLINE counter is_quiet is_debug has_failed

    counter=1

    _CMDLINE="$(cat /proc/cmdline)"

    # only live mode
    if ! echo "${_CMDLINE}" | LC_ALL=C grep -Fqs "boot=live"
    then
        exit 0
    fi

    # quiet
    if echo "${_CMDLINE}" | LC_ALL=C grep -Fq quiet ; then
        is_quiet=1
    fi

    # persistence
    if echo "${_CMDLINE}" | LC_ALL=C grep -Fq persistence ; then
        is_persistence=1
    fi

    # debug
    if echo "${_CMDLINE}" | LC_ALL=C grep -Fq debug ; then
        set -x

        is_debug=1
    fi

    # failsafe
    if echo "${_CMDLINE}" | LC_ALL=C grep -Fq failsafe ; then
        # we want failsafe or something went wrong, disable ecore_vsync
        export ECORE_NO_VSYNC=1
    fi

    # thanatests
    if echo "${_CMDLINE}" | LC_ALL=C grep -Fq "thanatests" ; then
        is_thanatests=1
    fi


    if [[ -n "$( cat /etc/debian_version | grep -E "(^7\.|wheezy)" )" ]] ; then
        is_wheezy=1
    fi

    # messages
    local message_sudo
    message_sudo="$( printf "$( eval_gettext "Run '%s' to become root, the default password is %s" )" "__sudo su__" "__elive__" )"

    local message_report
    # XXX: do not enable the next line using that message, ZSH bugs and cannot launch the graphical system using it
    #message_report="$( printf "$( eval_gettext "If you consider that there 's a bug in Elive or something can be improved, report it to %s" )" "http://bugs.elivecd.org" )"
    message_report="$( printf "$( eval_gettext "If you believe there is a bug in Elive or that something can be improved, report it to %s" )" "http://bugs.elivecd.org" )"


    local message_startx
    message_startx="$( printf "$( eval_gettext "Run '%s' to launch the graphical system" )" "__startx__" )"

    local message_ram_usage
    _ram_used="$( el_ram_usage_show used )"
    message_ram_usage="$( printf "$( eval_gettext "Elive booted using '%s' of your RAM" )" "__${_ram_used} MB__" )"


    # }}}
    # delete replacement file {{{
    # just in case
    if [[ -s "/etc/profile.d/zz-live-config_xinit.sh" ]] ; then
        sudo rm -f /etc/profile.d/zz-live-config_xinit.sh 2>/dev/null
    fi
    if [[ -s "/etc/zsh/zprofile.d/zz-live-config_xinit.sh" ]] ; then
        sudo rm -f /etc/zsh/zprofile.d/zz-live-config_xinit.sh 2>/dev/null
    fi

    # - delete replacement file }}}
    # pre hooks
    if ((is_thanatests)) ; then
        # always update the installer to its last version, and also apt
        ( sudo apt-get update ; sudo apt-get install -y eliveinstaller-6g ) 2>/dev/null 1>&2 &
    fi

    # needed for persistence:
    if [[ -e /var/lib/dpkg/info/lightdm.list ]] ; then
        el_info "ignoring $(basename $0) because we have lightdm installed, if lightdm doesn't autostarts you must run: 'sudo systemctl enable lightdm.service ; sudo systemctl start lightdm.service'"
        exit
    fi

    # mark a valid reached state
    if ! ((is_thanatests)) ; then
        ( state_valid_reached & )
    fi


    if echo "${_CMDLINE}" | LC_ALL=C grep -qsE "(nox11|nox11autologin|nostartx|nogui)" ; then
        echo -e ""

        #if ((is_wheezy)) ; then
            # change to default TTY1 instead of still on TTY7 (dialog's switches to it)
            if [[ "$(tty)" = "/dev/tty7" ]] ; then
                chvt 1
            fi

        #fi

        # no automatic X11 login, so show the help instead
        if [[ -z "$TMUX" ]] ; then
            echo -e ""
            echo -e ""
            #clear

            if sudo nvidia-privative-drivers-install -d 1>/dev/null 2>&1 ; then
                el_explain 0 "Nvidia card found, run 'sudo nvidia-privative-drivers-install' to install the nonfree drivers"
            fi
            #if sudo fglrx-privative-drivers-install -d 1>/dev/null 2>&1 ; then
                #el_explain 0 "ATI card found, run 'sudo fglrx-privative-drivers-install' to install the nonfree drivers"
            #fi

            el_explain 0 "$message_ram_usage"
            echo -e ""
            el_explain 0 "$message_sudo"
            el_explain 0 "$message_startx"
            echo -e ""
        fi
    else
        # tty must be only 1 or we will get multiple instances of X launched
        if [[ -z "${DISPLAY}" ]] && [[ "$(tty)" = "/dev/tty1" ]] ; then

            # stop plymouth because we want to show a dialog, switch to vt7 just in case
            #if ((is_wheezy)) ; then
            #    plymouth --ping 1>/dev/null 2>&1  && plymouth --hide-splash && chvt 1 && chvt 7
            #else
            #    plymouth --ping 1>/dev/null 2>&1  && plymouth --hide-splash && chvt 1
            #fi

            # exit plymouth
            sudo plymouth quit  1>/dev/null 2>&1
            waitfor plymouth

            # clean the tty
            #if ! ((is_debug)) ; then
                #reset ; clear
            #fi
            clear

            # set some needed variables
            export XDG_SESSION_TYPE="X11"
            # disable it directly from live mode
            export NO_AT_BRIDGE=1


            # we need to wait our needed tasks to finish, which in systemd are simply run in BG:
            #if [[ -e "/var/lib/dpkg/info/systemd.list" ]] && [[ -e "/var/lib/dpkg/info/live-config-systemd.list" ]] ; then
                #wait_for_remaining_processes
            #fi

            # wait for persistence-elive to finish if still running
            if ((is_persistence)) ; then
                if pidof -qx persistence-elive ; then
                    while true ; do
                        if ! pidof -qx persistence-elive ; then
                            break
                        fi
                        sync
                        LC_ALL=C sleep 0.5
                    done
                fi
            fi

            vgacard="$( lspci | grep -E "(VGA|3D|2D|Display)" )"
            if [[ -z "$vgacard" ]] ; then
                vgacard="$( sudo hwinfo --gfxcard 2>&1 | colors-remove | grep -E "(Model:|Vendor:|Device:|Modules:|Driver|Config)" )"
            fi

            # info about release status
            if ! ((is_wheezy)) && ! ((is_persistence)) ; then
                info_release
            fi

            # buster only suggested
            if ! ((is_wheezy)) ; then
                select_windowmanager
            fi

            # trusted system?
            if [[ "$MACHINE_VIRTUAL" != "yes" ]] && grep -Fqs "UNTRUSTED-OS" /proc/cmdline ; then
                local message_unauthenticated_elive
                #message_unauthenticated_elive="$( printf "$( eval_gettext "This system seems to have been altered. If you downloaded it from the website of Elive you need to record it with the suggested tools to your USB/DVD in order to have the system unaltered. But if you downloaded Elive from an external source / website it is VERY IMPORTANT to make sure its original contents have not been modified because this means a high-security risk for you, always download Elive from its original website to keep you safe." )" "" )"
                # note: the message needs to be this one: because is already-known that the system is untrusted
                message_unauthenticated_elive="$( printf "$( eval_gettext "This system does not seem to be an official version of Elive, or your USB/DVD was not created using the correct tools. Please download Elive only from the official website and use the recommended tools for its creation. It is your responsibility if you choose to continue using this system, and we have no control or knowledge of any changes happened here. It is crucial that you understand and accept these facts." )" "" )"
                dialog --timeout 30 --clear --colors --backtitle "Elive Systems: Use Ctrl-L to redraw, Enter to confirm" \
                    --title "***  WARNING - Elive Trusted System  ***" \
                    --msgbox "$message_unauthenticated_elive" \
                    0 0 || true
            fi


            # infinite loooop !
            while true
            do
                unset has_failed
                rm -f "/tmp/.X11-error-logs" 1>/dev/null 2>&1
                rm -f "$HOME/.local/share/xorg/Xorg.0.log" 1>/dev/null 2>&1
                rm -f "$HOME/.e-log.log" 1>/dev/null 2>&1


                # always update needed values for new startx
                source /etc/default/locale || true

                if grep -qsE "simulate=(ecore_no_vsync|ecorenovsync)" /proc/cmdline ; then
                    export ECORE_NO_VSYNC=1
                    el_info "Simulating: ECORE_NO_VSYNC=1"
                    sleep 2
                fi

                # simulation mode: fail only once
                if grep -Fqs "simulate=x11fail" /proc/cmdline && ! ((is_attempts)) ; then
                    has_failed=1
                else


                    #
                    # start graphical system
                    #

                    # wait for precache's
                    wait

                    # Note: startx should be already set to use the defined WM which is /etc/alternatives/x-window-manager
                    # we need to run it without arguments in order to run specific .xinitrc things like for the case of nvidia in optimus (full/only-nvidia) mode
                    if ((is_quiet)) ; then
                        #startx "$windowmanager" > /dev/null 2>/tmp/.X11-error-logs  || has_failed=1
                        startx > /dev/null 2>/tmp/.X11-error-logs  || has_failed=1
                    else
                        #startx "$windowmanager" 2>/tmp/.X11-error-logs  || has_failed=1
                        startx 2>/tmp/.X11-error-logs  || has_failed=1
                        sleep 1
                    fi
                fi

                # count a new one
                if ! ((has_failed)) ; then
                    counter="$(( $counter + 1 ))"
                    el_explain 3 "startx was run __${counter}__ times"
                fi


                # we have previously tried to make the desktop work without success, so repurt a correct status after that
                if ((is_attempts)) && ! ((has_failed)) ; then
                    el_warning "SUCCESS: desktop launched correctly after our previous attempts"
                    sleep 1
                fi


                # Failed attempts, try again:
                if ((has_failed)) ; then

                    # show error messages in console {{{
                    # note: the errors are returned from startx but not from the window manager, so if crashes we cannot know it from this level, we need to do it from the wrapper

                    # show possible error mesages to console
                    grep -iE "(error|parse|fatal|section)" /tmp/.X11-error-logs 1>&2
                    # small pause
                    LC_ALL=C  sleep 0.2
                    logs_errors="$( cat "/tmp/.X11-error-logs" "$HOME/.local/share/xorg/Xorg.0.log" | grep -E "^\[.*\] \((EE|WW)\) [[:alnum:]]+" )"

                    # - show error messages in console }}}
                    # specific erros / hardware {{{
                    # SQUASHFS errors
                    if ((is_reported_squashfs)) ; then
                        true
                    else
                        if echo "$logs_errors" | grep -qsE "SQUASHFS.*(error|Unable to read|read.*block)" || sudo -H dmesg | grep -qsiE "(SQUASHFS.*error|I/O error.*dev loop|I/O error.*sector|critical medium error.*sector|Medium Error)" ; then
                            echo ""
                            NOREPORTS=1 el_error "$( eval_gettext "Your device isn't reading Elive properly, which means the download might have been interrupted and damaged. Download Elive again and check its MD5 to make sure it’s correct, and use a good quality USB drive to prevent hardware errors that often happen with cheap ones." )"

                            # tell the user to use USB instead of DVD
                            if sudo -H dmesg | grep -qsiE "(critical medium error, dev sr0, sector|sr.*Medium Error|sr.*uncorrectable error|I/O error)" ; then
                                el_info "$( eval_gettext "Suggestion: Booting Elive from a USB thumb drive is faster and more stable than using a DVD" )"
                            fi

                            el_info "Press Enter to continue..."
                            read nothing
                        fi

                        is_reported_squashfs=1
                    fi

                    # kernels
                    if ((is_reported_kernel)) ; then
                        true
                    else
                        if [[ "$( ls -1 /boot/vmlinuz* | wc -l )" -gt 1 ]] ; then
                            el_info "$( eval_gettext "Your graphical system is not launching correctly. You can try booting with the other kernel version from the boot which includes different drivers. Also, consider trying other boot options." )"
                            el_info "Press Enter to continue..."
                            read nothing

                            #is_reported_kernel=1
                        #else
                            ## update: useless if we simply report to try with the other kernel
                            #if grep -Fqs "vmlinuz2" /proc/cmdline ; then
                                #el_info "$( eval_gettext "Your graphical system doesn't launch correctly. You are using the old Kernel. We suggest trying to use the newer one which includes newer drivers." )"
                                #el_info "Press Enter to continue..."
                                #read nothing
                            #fi

                            #is_reported_kernel=1
                        fi

                        is_reported_kernel=1
                    fi

                    # Different boot option
                    if ((is_reported_bootmode_others)) ; then
                        true
                    else
                        if grep -Fqs "bootmode=" /proc/cmdline ; then
                            el_info "$( eval_gettext "The graphical system fails to start. Try different boot options or boot in Console mode to execute manual commands, configure settings, or install required packages. Please share any improvements you make in our forums to enhance compatibility for everyone." )"
                            el_info "Press Enter to continue..."
                            read nothing
                        fi

                        is_reported_bootmode_others=1
                    fi

                    # ATI / AMD cards
                    if ((is_reported_ati)) ; then
                        true
                    else
                        if lspci | grep -E "\W+(VGA|3D|2D|Display)\W+" | grep -qsE "(AMD/ATI|Radeon)" ; then
                            el_info "$( eval_gettext "The graphical system doesn't start with an ATI graphic card. Try installing additional packages. Please share with Elive any modification improvements for better compatibility for everybody." )"

                            ask_for_console
                        fi

                        is_reported_ati=1
                    fi

                    # VirtualBox machines:
                    if ((is_reported_vboxvideo)) ; then
                        true
                    else
                        if lspci | grep -E "\W+(VGA|3D|2D|Display)\W+" | grep -qsE "(InnoTek|VirtualBox)" ; then
                            el_info "$( eval_gettext "The graphical system is unable to start in your Virtual Machine. Which is known to work correctly. You must reconfigure your virtual machine's graphical settings." ) VMSVGA, enable 3D, 1 core Cpu or APIC, KVM virtualization"

                            el_info "Press Enter to continue..."
                            read nothing
                        fi

                        is_reported_vboxvideo=1
                    fi

                    # virtual environments
                    if ((is_reported_parallels_or_virtual)) ; then
                        true
                    else
                        #if lspci | grep -E "\W+(VGA|3D|2D|Display)\W+" | grep -qsiE "(Parallels|Virtual)" ; then
                        if [[ "$MACHINE_VIRTUAL" = "yes" ]] ; then
                            el_info "$( eval_gettext "Graphical issues found in your virtual environment. Shut down your virtual machine and adjust your graphics settings, such as disabling 3D acceleration or using a different graphical driver." )"

                            el_info "Press Enter to continue..."
                            read nothing
                        fi

                        is_reported_parallels_or_virtual=1
                    fi

                    # - specific erros / hardware }}}

                    # remove nvidia conf {{{
                    if ((is_tried_nvidia)) ; then
                        sudo mv -f /tmp/20-nvidia.conf /etc/X11/xorg.conf.d/ 2>/dev/null || true
                    else
                        if [[ -s /etc/X11/xorg.conf.d/20-nvidia.conf ]] ; then
                            el_explain 0 "startx failed, let's try to remove our nvidia conf..."
                            sleep 2
                            echo -e "removed nvidia default (our) conf to see if this solves it" > "/tmp/.X11-error-logs-to-user"

                            sudo mv -f /etc/X11/xorg.conf.d/20-nvidia.conf /tmp/
                            is_tried_nvidia=1

                            el_warning "Graphical system failed to run, attempting with:  removing nvidia configuration"
                            is_attempts=1

                            continue
                        fi
                    fi

                    # - remove touchpad conf }}}
                    # DRM paused state {{{
                    # TODO: parse logs to know the cause and then
                    if ((is_tried_drm_paused)) ; then
                        el_info "already tried the drm paused state stuff"

                    else
                        if echo "$logs_errors" | grep -Fqs "systemd-logind returned paused fd for drm node" ; then

                            el_warning "Graphical system failed to run due to systemd-LOGIND"
                            is_tried_drm_paused=1
                            is_attempts=1

                            # reports:
                            if [[ -e "/dev/dri/card0" ]] ; then
                                el_warning "lsof /dev/dri/card0:\n$( sudo lsof /dev/dri/card0 )\n"
                            fi

                            # try by restarting logind
                            sudo -H systemctl restart systemd-logind
                            sync

                            # reports:
                            if [[ -e "/dev/dri/card0" ]] ; then
                                el_warning "lsof /dev/dri/card0 AFTER: $( sudo lsof /dev/dri/card0 )\n"
                            fi

                            if startx 2>/tmp/.X11-error-logs  ; then
                                el_warning "SUCCEED with restarting logind"
                                unset has_failed
                                continue
                            fi

                            # failed, try with another thing:
                            killall -9 plymouth 1>/dev/null 2>&1
                            killall -9 plymouthd 1>/dev/null 2>&1
                            sleep 2

                            if startx 2>/tmp/.X11-error-logs  ; then
                                el_warning "SUCCEED with restarting killing plymouth"
                                unset has_failed
                                continue
                            fi

                            # failed, try with another thing:
                            sync
                            sleep 10
                            if startx 2>/tmp/.X11-error-logs  ; then
                                el_warning "SUCCEED by waiting 10 seconds"
                                unset has_failed
                                continue
                            fi

                            continue
                        fi
                    fi

                    # - DRM paused state }}}
                    # ecore no vsync {{{
                    # remove ecore_vsync to test:
                    if ! [[ "${ECORE_NO_VSYNC}" = "1" ]] && [[ -x "$( which enlightenment_start )" ]] ; then
                        export ECORE_NO_VSYNC=1

                        echo -e "We have needed to run your desktop without ECORE VSYNC in order to make it working, we don't know if you could install Elive on this way without problem but is important to inform us about this problem so we can know how to solve it for you and everybody, report the issue in our website. You can also try to not use composite or to use it in software mode only." > "/tmp/.X11-error-logs-to-user"

                        el_warning "Graphical system failed to run, attempting with:  forcing ECORE_NO_VSYNC=1"
                        is_attempts=1

                        continue
                    fi

                    # - ecore no vsync }}}
                    # graphical specific conf {{{

                    if [[ -s "/tmp/xorg.conf" ]] || [[ -s "/etc/X11/xorg.conf" ]] ; then
                        if ((is_tried_graphical_conf_removed)) ; then
                            sudo mv -f /tmp/xorg.conf /etc/X11/xorg.conf 2>/dev/null || true
                        else
                            el_explain 0 "startx failed, let's try to remove our own configured 'xorg.conf' file..."
                            #sleep 2
                            #echo -e "We have needed to remove the touchpad configuration in order to make your graphical system able to run, this is not something expected so maybe this configuration is wrong with some computers like yours, report this issue to Elive from the Issues website so that we can understand the problem and fix it for everybody else affected. Thank you. Note: your system is able to install and perfectly work now." > "/tmp/.X11-error-logs-to-user"

                            sudo mv -f /etc/X11/xorg.conf /tmp/
                            is_tried_graphical_conf_removed=1

                            el_warning "Graphical system failed to run, attempting with: removing own xorg.conf file"
                            if lspci | grep -qsiE "(VGA|3D|Display).*(Silicon Integrated Systems|SiS).*671.*" ; then
                                el_warning "ooooooo You are using a SIS / 671 graphic card which is not very tested oooooo"
                            fi
                            is_attempts=1

                            continue
                        fi
                    fi

                    # - graphical specific conf }}}
                    # remove touchpad conf {{{
                    # remove our touchpad conf
                    if ((is_tried_touchpad)) ; then
                        sudo mv -f /tmp/80-eliveconfs-touchpad.conf /etc/X11/xorg.conf.d/ 2>/dev/null || true
                    else
                        el_explain 0 "startx failed, let's try to remove our touchpad confs..."
                        sleep 2
                        echo -e "We have needed to remove the touchpad configuration in order to make your graphical system able to run, this is not something expected so maybe this configuration is wrong with some computers like yours, report this issue to Elive from the Issues website so that we can understand the problem and fix it for everybody else affected. Thank you. Note: your system is able to install and perfectly work now." > "/tmp/.X11-error-logs-to-user"

                        sudo mv -f /etc/X11/xorg.conf.d/80-eliveconfs-touchpad.conf /tmp/
                        is_tried_touchpad=1

                        el_warning "Graphical system failed to run, attempting with:  removing touchpad configurations"
                        is_attempts=1

                        continue
                    fi

                    # - remove touchpad conf }}}
                    # DCC auto configure {{{
                    if [[ -x "$( sudo which get-edid )" ]] ; then
                        if ((is_tried_dcc)) ; then
                            # if we have already tried this remove it because is not a good idea to keep a static conf like this one
                            # TODO: example xorg error: ModeLine dotclock expected
                            sudo rm -f /etc/X11/xorg.conf.d/dcc-edid-monitor.conf
                        else
                            if timeout 25 sudo bash -c "get-edid 2>/dev/null | parse-edid 2>/dev/null >/etc/X11/xorg.conf.d/dcc-edid-monitor.conf" ; then
                                is_tried_dcc=1

                                el_explain 0 "Trying to generate a __DCC edid__ specific monitor configuration for your case"
                                sleep 2

                                el_warning "Graphical system failed to run, attempting with:  generated a DCC EDID configuration"
                                is_attempts=1

                                continue
                            fi
                        fi
                    fi

                    # - DCC auto configure }}}
                    # force SIS671 driver {{{
                    if echo "$vgacard" | grep -qsi "Silicon Integrated Systems" ; then
                        if ((is_tried_sis)) ; then
                            # if we have already tried this remove it because is not a good idea to keep a static conf like this one
                            sudo rm -f /etc/X11/xorg.conf.d/sis-mode.conf

                        else
                            sudo -H bash -c "cat > \"/etc/X11/xorg.conf.d/sis-mode.conf\"" << EOF
Section "Device"
    Identifier  "Default screen"
    Driver      "sis671"
    # append options here
EndSection
EOF

                            #
                            is_tried_sis=1

                            el_explain 0 "Forcing SIS graphical driver mode"
                            sleep 2

                            el_warning "Graphical system failed to run, attempting with:  using SIS mode"
                            is_attempts=1

                            continue
                        fi
                    fi

                    # - SIS }}}
                    # force MODESETTING mode {{{
                    if [[ -e "/dev/dri/card0" ]] ; then
                        if ((is_tried_modesetting)) ; then
                            # if we have already tried this remove it because is not a good idea to keep a static conf like this one
                            sudo rm -f /etc/X11/xorg.conf.d/modesetting-mode.conf

                        else
                            sudo -H bash -c "cat > \"/etc/X11/xorg.conf.d/modesetting-mode.conf\"" << EOF
Section "Device"
    Identifier  "Default screen"
    Driver      "modesetting"
    # append options here
EndSection
EOF

                            #
                            is_tried_modesetting=1

                            el_explain 0 "Forcing MODESETTING graphical driver mode"
                            sleep 2

                            el_warning "Graphical system failed to run, attempting with:  using MODESETTING mode"
                            is_attempts=1

                            continue
                        fi
                    fi

                    # - MODESETTING }}}
                    # force FBDEV mode {{{
                    if [[ -e "/dev/fb0" ]] ; then
                        if ((is_tried_fbdev)) ; then
                            # if we have already tried this remove it because is not a good idea to keep a static conf like this one
                            sudo rm -f /etc/X11/xorg.conf.d/fbdev-mode.conf

                        else
                            sudo -H bash -c "cat > \"/etc/X11/xorg.conf.d/fbdev-mode.conf\"" << EOF
Section "Device"
    Identifier  "Default screen"
    Driver      "fbdev"
    # append options here
EndSection
EOF

                            #
                            is_tried_fbdev=1

                            el_explain 0 "Forcing FBDEV graphical driver mode"
                            sleep 2

                            el_warning "Graphical system failed to run, attempting with:  using FBDEV mode"
                            is_attempts=1

                            continue
                        fi
                    fi

                    # - FBDEV }}}
                    # force VESA mode {{{
                    if ((is_tried_vesa)) ; then
                        # if we have already tried this remove it because is not a good idea to keep a static conf like this one
                        sudo rm -f /etc/X11/xorg.conf.d/vesa-mode.conf
                        sudo chmod u-s /usr/bin/xinit

                    else
                        sudo -H bash -c "cat > \"/etc/X11/xorg.conf.d/vesa-mode.conf\"" << EOF
Section "Device"
    Identifier  "Default screen"
    Driver      "vesa"
    # append options here
EndSection
EOF

                        # seems like this is needed in buster:
                        sudo chmod u+s /usr/bin/xinit

                        #
                        is_tried_vesa=1

                        el_explain 0 "Forcing VESA graphical driver mode"
                        sleep 2

                        el_warning "Graphical system failed to run, attempting with:  using VESA mode"
                        is_attempts=1

                        continue
                    fi

                    # - VESA }}}
                    # force VESA DCC mode {{{
                    if [[ -x "$( sudo which get-edid )" ]] ; then

                        if ((is_tried_vesa_dcc)) ; then
                            # if we have already tried this remove it because is not a good idea to keep a static conf like this one
                            sudo rm -f /etc/X11/xorg.conf.d/vesa-dcc-mode.conf
                        else
                            sudo rm -f /etc/X11/xorg.conf.d/vesa-dcc-mode.conf
                            timeout 25 sudo bash -c "get-edid 2>/dev/null | parse-edid 2>/dev/null >>/etc/X11/xorg.conf.d/vesa-dcc-mode.conf"
                            sudo -H bash -c "cat >> \"/etc/X11/xorg.conf.d/vesa-dcc-mode.conf\"" << EOF

Section "Device"
    Identifier  "Default screen"
    Driver      "vesa"
    # append options here
EndSection
EOF
                            #
                            is_tried_vesa_dcc=1

                            el_explain 0 "Forcing VESA + DCC graphical driver mode"
                            sleep 2

                            el_warning "Graphical system failed to run, attempting with:  using VESA mode + DCC EDID"
                            is_attempts=1

                            continue
                        fi
                    fi

                    # - VESA DCC }}}
                fi

                # TODO: some things useful to fix:
                #
                # - nvidia / fglrx deinstall?
                # -  - switch to previous version of nvidia?
                #       nvidia / ati should be installed from daemon, from an external command which can be run manually at any time, and so trying to install previous versions based on parameters too, etc
                #           - install from elive-tools, so we can call the tool from installed system too, like:  nvidia-privative-drivers-install
                # - switch to vesa?
                # - force resolution? (old crt monitors? no-dcc monitors like the one in hackership?)
                #    - graphical-configuration-suite full ? (from topaz deliver)
                # - TODO: implement some ^ auto-fixes from that tool too, + betatest!
                # - FIXME: actually the 30 % of users are unable to run the graphical system, let's see if after nvidia/fglrx the problem stills here, and on such case ask them where the problem is, maybe we -just- need the graphical-configuration-suite too
                # -
                # -




                # variable still existing after our fixups?
                # we have nothing to fix? ok, its -seriously- time to prompt the user
                if ((has_failed)) ; then
                    local message_graphical_problem
                    message_graphical_problem="$( printf "$( eval_gettext "The graphical system seems to have failed to launch correctly. Connect to our chat or forum on the Elive website for support. We need to know your specific issues to help you and other users. Use 'Shift + PageUp' to read previous messages. Search in Google if there's any specific configurations needed for your computer model." )" )"


                    # FIXME: implement & betatest
                    # FIXME: how much is worth? maybe we can just suggest to connect to the chat so we can help them, even remotely? (giving instructions)
                    #local message_remote_assistance
                    #message_remote_assistance="$( printf "$( eval_gettext "Do you want to contact now an Elive developer to fix the problem for you? he can try to make Elive work on your computer and in the same time he will understand where the problem is for solve it on Elive which could affect to other users too." )" "" )"


                    # show error messages {{{
                    if [[ -s "/tmp/.X11-error-logs" ]] ; then
                        el_explain 0 "STDERR from __startx__ command:"
                        cat /tmp/.X11-error-logs 1>&2
                    fi
                    if grep -Fqs "EE" /var/log/Xorg.0.log "$HOME/.local/share/xorg/Xorg.0.log" ; then
                        el_explain 0 "Error messages from __Xorg logs__:"
                        grep -F "EE" /var/log/Xorg.0.log "$HOME/.local/share/xorg/Xorg.0.log" | tail -n +6
                    fi


                    # - show error messages }}}
                    # warn the user
                    el_explain 0 "__Elive Graphical Startup__"

                    echo -e "\n${message_graphical_problem}\n"

                    # eliveremote?
                    #echo -e "${message_remote_assistance}"
                    #if el_confirm "Launch eliveremote now?" ; then
                        #eliveremote
                    #fi

                    if sudo nvidia-privative-drivers-install -d 1>/dev/null 2>&1 ; then
                        el_info "$( eval_gettext "You have an Nvidia graphics card. You can reboot and select the Nvidia drivers option to use the official drivers from Nvidia." )"
                    fi

                    # report the graphic card which is problematic
                    echo ""
                    #el_warning "Unable to run graphical system for this type of drivers:\n$( sudo lspci -nnk | grep "VGA\|ED\|Display" -A2 )\nFrom a computer model of:\n  Manufacturer '$( sudo dmidecode -s system-manufacturer )'\n  System Version '$( sudo dmidecode -s system-version)'\n  Product name '$( sudo dmidecode -s system-product-name )'\nPLEASE REPORT THE ISSUE TO THE ELIVE FORUMS so we can fix it"



                    el_error "\nSTARTX ERRORS:\n$logs_errors \n\nDMESG:\n$(sudo dmesg | grep -Fi error | tail -n 6)\n\nCONF FILES: $(find /etc/X11/xorg.conf.d/ -type f | sed -e 's|/etc/X11/xorg.conf.d/||g' | tr '\n' ' ')\n\nVGA card:\n${vgacard}\n\nPROC: $(cat /proc/cmdline)\n\nMODULES:\n$(lsmod | grep -iE "(nvidia|radeon|r1|amd|nouveau|trident|sis)" )\n\nSPLASH state: $( if pidof plymouth || pidof plymouthd ; then echo "plymouth running" ; else echo "plymouth not running" ; fi )\n\nDRI state: $( lsof /dev/dri/card0 )\n$( sudo lspci -nnk | grep "VGA\|ED\|Display" -A2 )\n\nFrom a computer model of:\n  Manufacturer '$( sudo dmidecode -s system-manufacturer )'\n  System Version '$( sudo dmidecode -s system-version)'\n  Product name '$( sudo dmidecode -s system-product-name )'"
                    #el_info "Try to boot in console mode to see if you can run the graphical system yourself, and report us any improvement for Elive in our forums"

                    el_report_to_elive "Some XORG ERROR messages:\n$( grep -F EE "$HOME/.local/share/xorg/Xorg.0.log" 2>&1 | tail -n 20 )"

                    NOREPORTS=1 el_warning "Unable to run graphical system for this type of drivers:\n${vgacard}"
                    echo ""


                    # pause so the user can read or run graphical system again
                    #clear
                    echo
                    ask_for_console
                fi


                # small pause, just in case we want to do something
                if ! ((is_quiet)) ; then
                    sleep 2
                fi
                # if debug or multiple attempts, ask to the user what to do
                if ((is_debug)) || [[ "$counter" -gt 3 ]] ; then
                    #clear
                    echo
                    ask_for_console
                fi


                echo -e ""
                echo -e "" 1>&2
                el_explain 0 "Restarting the graphical interface... ($counter)"

                sudo killall -9 enlightenment_system 2>/dev/null 1>&2 || true
                sudo killall -9 enlightenment 2>/dev/null 1>&2 || true
                sudo killall -9 enlightenment_start 2>/dev/null 1>&2 || true
                # sudo killall -9 efreetd 2>/dev/null 1>&2 || true
                sudo killall -9 tumblerd 2>/dev/null 1>&2 || true

                # cleanups to not open new different DISPLAY values
                rm -rf /tmp/.X11-unix/* /tmp/.X*lock 2>/dev/null || true

                LC_ALL=C sleep 3

            done
        fi
    fi

}

#
#  MAIN
#
main "$@"

# vim: set foldmethod=marker filetype=sh :
